package projet_these.es;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.text.DecimalFormat;
import java.util.Arrays;
import java.awt.List;
import projet_these.et.IslandModel;
import java.util.StringTokenizer;
import projet_these.et.Population;

/**
 * A set of static tools to save out datas in files
 * @author Gregory Deceliere - UMR CNRS 5558 - "BBE"
 * @version 01/03/02
 */
public class OutPutDataFiles
{

    /**
     * Whrite the file header with the populations number spaced by a tab
     * @param model IslandModel
     * @param outputs FleWriter
     * @throws java.io.IOException writing exceptions
     */
    public static void outputFileHeader(IslandModel model, FileWriter outputs)
            throws IOException
    {

        for (int i = 0; i < model.nb_pop; i++) {
            if (i != model.nb_pop - 1) {
                outputs.write("\t" + (i + 1));
            } else {
                outputs.write("\t" + (i + 1) + "\n");
            }
        }
    }

    /**
     * Procedure writing the mean for each population
     * @param tmp int
     * @param model IslandModel
     * @param outputs FileWriter
     * @param df DecimalFormat
     * @throws java.io.IOException the exceptions
     */
    public static void outputMean(int tmp, IslandModel model,
                            FileWriter outputs,
                            DecimalFormat df) throws IOException
    {
        for (int i = 0; i < model.nb_pop; i++) {

            /* Si c'est la premiere colonne on ecrit
                         le pat de temps puis la moyenne*/
            if (i == 0) {
                outputs.write(df.format((double) tmp) + "\t" +
                              df.format((double) model.iles[i].getAllCopiesNb(
                        tmp)));
            }

            /* Si ce n'est pas la premiere ni la derniere
                         colonne on ecrit uniquement la moyenne*/
            if (i > 0 && i < model.nb_pop - 1) {
                outputs.write("\t" +
                              df.format((double) model.iles[i].getAllCopiesNb(
                        tmp)));
            }

            /* Si c'est la derniere colonne on ecrit
                         la moyenne puis on retourne  la ligne*/
            if (i == model.nb_pop - 1) {
                outputs.write("\t" +
                              df.format((double) model.iles[i].getAllCopiesNb(
                        tmp)) +
                              "\n");
            }
        }
    }

    /**
     * Procedure writing the insertion profile for each population
     * @param outputs FileWriter
     * @param tmp int
     * @param df DecimalFormat
     * @throws java.io.IOException the exceptions
     */
    public static void outputProfile(int tmp, IslandModel model,
                                     FileWriter outputs,
                              DecimalFormat df) throws IOException
    {
        // Profile courant
        int[] table1d;

        // Tous les profiles concatenes et sommes
        float[][] table2d = new float[model.nb_pop][120 * 3];

        // Enregistrement des profiles
        for (int j = 0; j < model.nb_pop; j++) {
            model.iles[j].setIns_Profile(tmp);
        }

        // Calcul et ecriture des profiles en fin de procedure
        if (tmp == model.tps) {
            // Calcul
            for (int i = 0; i < model.nb_pop; i++) {
                Arrays.fill(table2d[i], 0f);
                for (int j = 0; j < model.iles[i].ins_profile.size(); j++) {
                    for (int k1 = 0; k1 < 3; k1++) {
                        table1d = ((int[][]) model.iles[i].ins_profile.elementAt(j))[
                                  k1];
                        for (int k2 = 0; k2 < 120; k2++) {
                            table2d[i][k1 * 120 + k2] += (float) table1d[k2];
                        }
                    }
                }
                for (int k = 0; k < 120 * 3; k++) {
                    table2d[i][k] /= ((float) model.iles[i].ins_profile.size());
                    table2d[i][k] /= ((float) (model.iles[i].taille * 2));
                }
            }

            // Ecriture
            for (int k = 0; k < 120 * 3; k++) {
                for (int i = 0; i < model.nb_pop; i++) {
                    if (i == 0) {
                        outputs.write((k + 1) + "\t" + df.format((double)
                                table2d[i][k]));
                    }
                    if (i > 0 && i < model.nb_pop - 1) {
                        outputs.write("\t" + df.format((double)
                                table2d[i][k]));
                    }
                    if (i == model.nb_pop - 1) {
                        outputs.write("\t" + df.format((double)
                                table2d[i][k]) + "\n");
                    }
                }
            }
        }
    }

    /**
     * Procedure writing the var for each population during the
     * equilibrium period
     * @param tmp int
     * @param model IslandModel
     * @param outputs FileWriter
     * @param df DecimalFormat
     * @throws java.io.IOException the exceptions
     */
    public static void outputVar(int tmp, IslandModel model, FileWriter outputs,
                          DecimalFormat df) throws IOException
    {
        for (int j = 0; j < model.nb_pop; j++) {
            model.iles[j].variance(tmp);

            if (j == 0) {
                outputs.write(df.format((double) tmp) + "\t" +
                              df.format((double) model.iles[j].var_nb_cop[tmp
                                        + model.iles[j].var_nb_cop.length -
                                        model.iles[j].nb_cop.length]));
            }
            if (j > 0 && j < model.nb_pop - 1) {
                outputs.write("\t" +
                              df.format((double) model.iles[j].var_nb_cop[tmp
                                        + model.iles[j].var_nb_cop.length -
                                        model.iles[j].nb_cop.length]));
            }
            if (j == model.nb_pop - 1) {
                outputs.write("\t" +
                              df.format((double) model.iles[j].var_nb_cop[tmp
                                        + model.iles[j].var_nb_cop.length -
                                        model.iles[j].nb_cop.length])
                              + "\n");
            }
        }
    }

    /**
     * saveParameters
     * Save the parameters of a model
     * @param model IslandModel
     * @param file File
     */
    public static void saveParameters(IslandModel model, File file)
            throws IOException
    {
        FileWriter file_writer = new FileWriter(file);
        // Ecriture du model de contamination
        file_writer.write(model.conta_mod + "\n");

        // Ecriture du nombre de generations
        file_writer.write(model.tps + "\n");

        // Ecriture du nombre de populations
        file_writer.write(model.nb_pop + "\n");

        /* Ecriture des differentes tailles de populations:
                     - dans le cas de tailles homogenes
                     - dans le cas de tailles heterogenes
         */
        file_writer.write(model.rep_tailles + "\n");

        // Cas de tailles homogenes
        if (model.rep_tailles.equals("H")) {
            file_writer.write(model.N[0] + "\n");
        }

        // Cas de tailles heterogenes
        if (model.rep_tailles.equals("NH")) {
            /* Dans le cas de tailles heterogenes, le separateur utilise est
              la tabulation*/
            for (int i = 0; i < model.nb_pop - 1; i++) {
                file_writer.write(model.N[i] + "\t");
            }
            file_writer.write(model.N[model.nb_pop - 1] + "\n");
        }

        /* Ecriture du nombre
                     moyen de copies par population*/
        for (int i = 0; i < model.nb_pop - 1; i++) {
            file_writer.write(Float.toString(model.nbcop[i]) + "\t");
        }
        file_writer.write(Float.toString(model.nbcop[model.nb_pop - 1]) + "\n");

        // Traitement du cas d'une explosion locale
        if (model.conta_mod.equals("LB")) {
            saveParametersLB(model, file_writer);
        }
        // Traitement du cas d'une copie agressive
        else if (model.conta_mod.equals("AC1") ||
                 model.conta_mod.equals("AC2")) {
            saveParametersAC1(model, file_writer);
        }

        // Type de migration
        file_writer.write(model.rep_tx + "\n");

        // Cas d'une diffusion homogene
        if (model.rep_tx.equals("H"))
            file_writer.write(Float.toString(model.tx_migration[1][0]) + "\n");

        // Cas d'une diffusion heterogene
        if (model.rep_tx.equals("NH")) {
            for (int i = 1; i < model.nb_pop; i++) {
                for (int j = 0; j < i - 1; j++) {
                    file_writer.write(Float.toString(model.tx_migration[i][j])
                                      + "\t");
                }
                file_writer.write(Float.toString(model.tx_migration[i][i - 1])
                                      + "\n");
            }
        }

        file_writer.close();

    }

    /**
     * saveParametersLB()
     * Save the parameters of a LB model
     * @param model IslandModel
     * @param file_writer FileWriter
     */
    public static void saveParametersLB(IslandModel model,
        FileWriter file_writer) throws IOException
    {
    }

    /**
     * saveParametersAC1
     * Save the parameters of an AC1 model
     * @param model IslandModel
     * @param file_writer FileWriter
     */
    public static void saveParametersAC1(IslandModel model,
        FileWriter file_writer) throws IOException
    {
    }
    /**
     *return the list of outputdatafiles of the same simul name of the .para 
     * file in its same directory
     */
    public static File[] checkExistingOutputFiles(File para_file){
        File[] files = para_file.getParentFile().listFiles();
        files = null;
        return files;
    }
}
