package projet_these.et;

import java.io.*;
import java.text.DecimalFormat;
import java.util.Arrays;
import java.util.Vector;

import projet_these.rn.bnldev;
import projet_these.es.OutPutDataFiles;




/**
 * Implementation of set of Island
 * @author Gregory Deceliere - UMR CNRS 5558 - "BBE"
 * @version 01/03/02
 */
public class IslandModel implements Serializable {
    
    /**
     * Number of Island
     */
    public int nb_pop;
    
    /**
     * Array of average numbers of copies in each Island in the beginning
     */
    public float[] nbcop;
    
    /**
     * Array of average numbers of agressive copies in each Island in the beginning
     */
    public float[] nbcop2;
    
    /**
     * Migration Matrix
     */
    public float[][] tx_migration;
    
    /**
     * Under Migration Matrix excluding the diagonal, i.e. having dimension
     * nb_pop*(nb_pop-1)
     */
    public int[][] imigration;
    
    /**
     * Neighborhood graph
     */
    public Population[][] voisinage;
    
    /**
     * Array of Islands
     */
    public Population[] iles;
    
    /**
     * Array of each Island size
     */
    public int[] N;
    
    /**
     * Hot Spots
     */
    public boolean[] hot_spots;
    
    /**
     * Simulation duration in generation
     */
    public int tps;
    
    /**
     * Proporty of homogeneity of population sizes
     */
    public String rep_tailles;
    
    /**
     * Proporty of homogeneity of migration rates
     */
    public String rep_tx;
    
    /**
     * Contamination model
     */
    public String conta_mod;
    
    /**
     * Input File name
     */
    public String nomFic1 = "";
    
    /**
     * Output File name
     */
    public String nomFic2;
    
    /**
     * Backup file of the current object
     */
    public File modelFile;
    
    /**
     * Activation of the Console mode
     */
    public boolean console;
    
    
    /**
     * Contructor of initialization of the model
     */
    public IslandModel() {
        // Initialisation du nombre de populations
        nb_pop = 1;
        
        // Initialisation du tableau des nombres moyens de copies initiaux
        nbcop = new float[nb_pop];
        nbcop[0] = -1;
        
        // Initialisation de la matrice de migration
        tx_migration = new float[nb_pop][nb_pop];
        tx_migration[0][0] = -1f;
        
        // Initialisation de la matrice d'imigration
        imigration = new int[nb_pop][];
        
        // Initialisation du graphe de voisinage
        voisinage = new Population[nb_pop][];
        
        // Initialisation du tableau des populations
        iles = new Population[nb_pop];
        
        // Initialisation du tableau des tailles de chaque population
        N = new int[nb_pop];
        N[0] = -1;
        
        // Initialisation de la duree simulee (en generation)
        tps = 0;
        
        /* Initialisation des proprietes d'homogeneite des tailles de populations
                                 et des taux de migration*/
        rep_tailles = "";
        rep_tx = "";
    }
    
    
    
    
    
    
    /**
     * Procedure editing entry parameters of the model
     * @return a string representing the set of parameters with explicit comments
     */
    public String getInfo() {
        
        String situation = "";
        
        // Noms des fichiers d'entree et de sortie
        //situation +=
                //"Simulation name: " + this.nomFic1.substring(this.nomFic1.lastIndexOf(System.getProperty("file.separator"))+1, this.nomFic1.lastIndexOf(".para")) + "\n\n";// + "\nOutput file name: " +
        situation +=
                "Simulation name: " + this.nomFic1.substring(this.nomFic1.lastIndexOf(System.getProperty("file.separator"))+1) + "\n\n";
        //this.nomFic2 + "\n";
        
        /* Ecriture :
                - du temps de simulation
                - du nombre de populations
                - du parametre d'homogeneite des tailles
         */
        situation +=
                "Duration (in generation): " + tps + "\n\n" + "Number of populations: " +
                nb_pop + "\n\n" +
                "Homogeneity of populations sizes (H=homogeneous ; NH=otherwise): " +
                rep_tailles + "\n";
        
        // Ecriture de la (ou des) taille (s)
        if (rep_tailles.equals("H"))
            situation += "\tSize of all populations: " + N[0] + "\n\n";
        else {
            for (int i = 0; i < nb_pop; i++)
                situation += (i < nb_pop - 1) ?
                    "\tSize of population " + (i + 1) + ": " +
                        N[i] + "\n":
                    "\tSize of population " + (i + 1) + ": " +
                        N[nb_pop - 1] + "\n\n";
        }
        
        // Ecriture des nombre de copies moyens par population
        situation += "Copies numbers\n";
        for (int i = 0; i < nb_pop; i++)
            situation += (i < nb_pop - 1) ?
                "\tPopulation " + (i + 1) + ": " + nbcop[i] + "\n":
                "\tPopulation " + (i + 1) + ": " + nbcop[i] + "\n\n";
        
        /* Ecriture du (ou des) taux de migration ainsi que du parametre
        d'homogeneite associe */
        situation += "Homogeneity of migration (H=homogeneous ; NH=otherwise): ";
        if (rep_tx.equals("H"))
            situation +=
                    "H\n" + "\tNumber of migrants between two populations: " +
                    tx_migration[1][0] + "\n";
        else {
            situation += "NH\n" + "Absolute migration matrix\n\t";
            for (int i = 0; i < nb_pop - 1; i++)
                situation +=
                        (i < nb_pop - 2) ? (i + 1) + "\t": (i + 1) + "\n";
            for (int i = 1; i < nb_pop; i++) {
                situation += (i + 1) + "\t";
                for (int j = 0; j < i; j++)
                    situation += (j < i - 1) ? tx_migration[i][j] + "\t":
                        tx_migration[i][j] + "\n";
            }
        }
        
        // On retourne l'ensemble des parametre sous la forme d'un String
        return situation;
    }
    
    
    /**
     * Procedure editing entry parameters of the model (very similar to getInfo)
     * @return a string representing the set of parameters
     */
    public String toString() {
        
        String situation = "";
        
        /* Ecriture :
                - du temps de simulation
                - du nombre de populations
                - du parametre d'homogeneite des tailles
         */
        situation += tps + "\n" + nb_pop + "\n" + rep_tailles + "\n";
        
        // Ecriture de la (ou des) taille (s)
        if (rep_tailles.equals("H"))
            situation += N[0] + "\n";
        else {
            for (int i = 0; i < nb_pop; i++)
                situation += (i < nb_pop - 1) ? N[i] + "\t":
                    N[nb_pop - 1] + "\n";
        }
        
        // Ecriture des nombre de copies moyens par population
        for (int i = 0; i < nb_pop; i++)
            situation +=
                    (i < nb_pop - 1) ? nbcop[i] + "\t": nbcop[i] + "\n";
        
        /* Ecriture du (ou des) taux de migration ainsi que du parametre
        d'homogeneite associe */
        if (rep_tx.equals("H"))
            situation += "H\n" + tx_migration[1][0] + "\n";
        else {
            situation += "NH\n";
            for (int i = 1; i < nb_pop; i++) {
                for (int j = 0; j < i; j++)
                    situation += (j < i - 1) ? tx_migration[i][j] + "\t":
                        tx_migration[i][j] + "\n";
            }
        }
        
        // On retourne l'ensemble des parametre sous la forme d'un String
        return situation;
    }
    
    
    /**
     * Procedure setting an IslandModel
     * @param n    Number of islands
     * @param size Populations sizes
     * @param tx   Migration rate
     */
    public void setIslandModel(int n, int size, int tx) {
        
    }
    
    
    /**
     * Procedure setting an Stepping-Stone Model 2D
     * @param rows  Number of rows
     * @param lines Number of lines
     * @param size  Populations sizes
     * @param tx    Migration rate
     */
    public void setStepStoMod2D(int rows, int lines, int size, int tx) {
        
    }
    
    
    /**
     * Procedure setting the islands
     *
     *
     */
    public void setPopulations() {
        
        
        // Traitement du cas sans modle de contamination
        if (this.conta_mod.equals("NM")) {
            for (int i = 0; i < this.nb_pop; i++) {
                iles[i] = new Population(N[i], nbcop[i], tps + 1);
            }
        }
        
        // Traitement du cas d'une explosion locale
        else if (this.conta_mod.equals("LB")) {
            
            // Initialisation des iles
            for (int i = 0; i < this.nb_pop; i++) {
                iles[i] = new PopulationLocalBurst(N[i], nbcop[i],
                        hot_spots[i], tps + 1);
            }
        }
        
        // Traitement du cas d'une copie agressive
        else if (this.conta_mod.equals("AC1") ||
                this.conta_mod.equals("AC2")) {
            
            
            
            
            
            
            
            // Initialisation des iles
            float[] tab_cop = new float[2];
            for (int i = 0; this.conta_mod.equals("AC1")
            && i < this.nb_pop; i++) {
                tab_cop[0] = this.nbcop[i];
                tab_cop[1] = this.nbcop2[i];
                iles[i] = new PopulationPoly1(N[i], tab_cop, tps + 1);
            }
        }
        
    }
    
    /**
     * Procedure setting migration
     */
    public void setMigration() {
        // Indices de noeuds lies par un arc
        int i, j;
        
        // Plus petite taille d'un couple de populations
        int min_Nij;
        
        // Compteur utilise pour initialiser imigration
        int compteur = 0;
        
        // Matrice stochastique de migration
        int[][] mig_matrix = new int [nb_pop][nb_pop];
        for (i = 0; i < nb_pop; i++)
            mig_matrix[i][i] = N[i];
        
        // Ensemble des arc du graphe
        Vector arc_set = new Vector((int) (0.5f * ((nb_pop - 1) * nb_pop)));
        for (i = 1; i < nb_pop; i++) {
            for (j = 0; j < i; j++) {
                arc_set.add(compteur, new int[2]);
                ((int []) arc_set.elementAt(compteur))[0] = i;
                ((int []) arc_set.elementAt(compteur))[1] = j;
                compteur++;
            }
        }
        
        while(arc_set.size() > 0) {
            // Un arc tire au hasard
            int ra = (int) (Math.random() * arc_set.size());
            i = ((int []) arc_set.elementAt(ra))[0];
            j = ((int []) arc_set.elementAt(ra))[1];
            min_Nij = Math.min(mig_matrix[i][i], mig_matrix[j][j]);
            mig_matrix[i][j] = (min_Nij > 0) ?
                (int) bnldev.binom(tx_migration[i][j] / min_Nij, min_Nij) : 0;
            mig_matrix[j][i] = mig_matrix[i][j];
            mig_matrix[i][i] -= mig_matrix[i][j];
            mig_matrix[j][j] -= mig_matrix[j][i];
            arc_set.removeElementAt(ra);
        }
        
        for (i = 0; i < nb_pop; i++) {
            compteur = -1;
            for (j = 0; j < nb_pop; j++) {
                if (i != j) {
                    compteur += 1;
                    /* tx_migration[i][j] est le taux de migration
                      de j vers i (i.e. ligne i colonne j de la matrice
                      de migration
                     */
                    imigration[i][compteur] = mig_matrix[i][j];
                }
            }
            iles[i].setTaux_Migration(imigration[i]);
        }
    }
    
    
    /**
     * Procedure setting the network of islands
     */
    public void setNeightbourhood() {
        // Compteur utilise pour initialiser voisinage
        int compteur;
        
        for (int i = 0; i < nb_pop; i++) {
            compteur = -1;
            for (int j = 0; j < nb_pop; j++) {
                if (i != j) {
                    compteur += 1;
                    voisinage[i][compteur] = iles[j];
                }
            }
            iles[i].setVoisins(voisinage[i]);
        }
    }
    
    
    /**
     * Procedure simulating the model according to its paramaters
     * @param i the current simulation step
     */
    public void simulation(int i) {
        
        // Flux d'octet vers les fichiers de sorties en ecriture
        FileWriter sorties_mean, sorties_var, sorties_profile;
        
        // Nombre de generation avant calcul des frquences d'insertion
        int seuil = (this.tps >= 10)? this.tps - 10 : tps;
        
        // Decimal format use to format decimal numbers
        DecimalFormat df = new DecimalFormat("##0.00");
        
        try {
            // If first generation, overwrite an eventually existing file. After append.
            boolean append = (i == 0) ? false : true;
            sorties_mean = new FileWriter(nomFic2 + ".mea", append);
            sorties_var = new FileWriter(nomFic2 + ".var", append);
            sorties_profile = new FileWriter(nomFic2 + ".frq", append);
            
            // Creation de l'entete des fichier de sortie
            if (! this.conta_mod.equals("AC1") && i == 0) {
                OutPutDataFiles.outputFileHeader(this, sorties_mean);
                OutPutDataFiles.outputFileHeader(this, sorties_var);
                OutPutDataFiles.outputFileHeader(this, sorties_profile);
            }
            
            /* Ecriture des nombres moyens de copies  chaque pat de temps
            pour chaque population*/
            
            if ((! this.conta_mod.equals("AC1")) && i >= seuil) {
                OutPutDataFiles.outputProfile(i, this, sorties_profile, df);
            }
            
            OutPutDataFiles.outputMean(i, this, sorties_mean, df);
            OutPutDataFiles.outputVar(i, this, sorties_var, df);
            makeProcess(i);
            
            //Fermeture des flux
            sorties_mean.close();
            sorties_var.close();
            sorties_profile.close();
            
            
            
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    
    /**
     * Coordinates the process
     * @param tmp int the current line
     * @throws java.io.IOException the exceptions
     */
    public void makeProcess(int tmp) throws
            IOException {
        
        if (this.console) {
            System.out.print("t=" + tmp + "\r");
        }
        
        // Processus de simulation
        if (tmp < tps) {
            
            // Reproduction  l'interieur de chaque population
            for (int j = 0; j < nb_pop; j++) {
                iles[j].reproduction();
            }
            
            /* Migration dans des zones tampons pour eviter
                         les migration multiples
             */
            this.setMigration();
            for (int j = 0; j < nb_pop; j++) {
                iles[j].migration();
            }
            
            // Fusion des zones tampons avec le reste des populations
            for (int j = 0; j < nb_pop; j++) {
                iles[j].post_migration(tmp + 1);
            }
        }
    }
    
}
/**
 * Coordinates the process
 * @param tmp int the current line
 * @throws java.io.IOException the exceptions
 *
 * public void makeProcess(int tmp) throws
 * IOException {
 *
 * if (this.console) {
 * System.out.print("t=" + tmp + "\r");
 * }
 *
 * // Processus de simulation
 * if (tmp < tps) {
 *
 * // Reproduction  l'interieur de chaque population
 * for (int j = 0; j < nb_pop; j++) {
 * iles[j].reproduction();
 * }
 *
 * /* Migration dans des zones tampons pour eviter
 * les migration multiples
 *
 * this.setMigration();
 * for (int j = 0; j < nb_pop; j++) {
 * iles[j].migration();
 * }
 *
 * // Fusion des zones tampons avec le reste des populations
 * for (int j = 0; j < nb_pop; j++) {
 * iles[j].post_migration(tmp + 1);
 * }
 * }
 * }
 *
 * /**
 * Coordinates the process
 * @param tmp int the current line
 * @param outputs FileWriter the current output stream
 * @param df DecimalFormat
 * @param console boolean
 * @throws java.io.IOException the exceptions
 *
 * public void makeProcess(int tmp, FileWriter outputs,
 * DecimalFormat df, boolean console) throws IOException {
 *
 * if (console)
 * System.out.print("t=" + tmp + "\r");
 *
 * for (int j = 0; j < nb_pop; j++) {
 *
 * /* Si c'est la premiere colonne on ecrit
 * le pat de temps puis la moyenne*
 * if (j == 0)
 * outputs.write(df.format((double) tmp) + "\t" +
 * df.format((double) iles[j].getAllCopiesNb(tmp)));
 *
 * /* Si ce n'est pas la premiere ni la derniere
 * colonne on ecrit uniquement la moyenne*
 * if (j > 0 && j < nb_pop - 1)
 * outputs.write("\t" +
 * df.format((double) iles[j].getAllCopiesNb(tmp)));
 *
 * /* Si c'est la derniere colonne on ecrit
 * la moyenne puis on retourne  la ligne*
 * if (j == nb_pop - 1)
 * outputs.write("\t" +
 * df.format((double) iles[j].getAllCopiesNb(tmp)) + "\n");
 * }
 *
 * // Processus de simulation
 * if (tmp < tps) {
 *
 * // Reproduction  l'interieur de chaque population
 * for (int j = 0; j < nb_pop; j++)
 * iles[j].reproduction();
 *
 * /* Migration dans des zones tampons pour eviter
 * les migration multiples
 *
 * this.setMigration();
 * for (int j = 0; j < nb_pop; j++)
 * iles[j].migration();
 *
 * // Fusion des zones tampons avec le reste des populations
 * for (int j = 0; j < nb_pop; j++)
 * iles[j].post_migration(tmp + 1);
 * }
 * }
 * }
 *
 * /**
 * Procedure of parameters initialization by the input file
 */
/*    public void initierFic() {
 
        // Flux d'octets en lecture
        BufferedReader entree = null;
 
        // Chaine de caracteres marquee
        StringTokenizer st;
 
        // Exception de recherche d'un fichier
        FileNotFoundException exc;
 
        /* Intancie un tampon de lecture construit sur le fichier
        texte d'entree en lecture avec un do while
        de gestion des erreurs*/
 /*       do
        {
            exc = null;
            try {
                entree = new BufferedReader(new FileReader(nomFic1));
            } catch (FileNotFoundException excp) {
                exc = excp;
                System.out.println("File " + nomFic1 + " not found\n" +
                        "Enter a new input file name :");
                try {
                    nomFic1 = EntreesSorties.lireChaine();
                } catch (IOException e) {
                    System.out.println(e);
                }
            }
        }
        while (exc != null)
            ;
  
        // Instanciation des parametres separateur apres separateur
        try {
            // Saisie du model de contamination
            conta_mod = entree.readLine();
  
            // Saisie du nombre de generations
            tps = Integer.valueOf(entree.readLine()).intValue();
  
            // Saisie du nombre de populations
            nb_pop = Integer.valueOf(entree.readLine()).intValue();
  
            /* Saisie des differentes tailles de populations:
            - dans le cas de tailles homogenes
            - dans le cas de tailles heterogenes
  */
 /*           N = new int[nb_pop];
            rep_tailles = entree.readLine();
  
            // Cas de tailles homogenes
            if (rep_tailles.equals("H")) {
                N[0] = Integer.valueOf(entree.readLine()).intValue();
                for (int i = 1; i < nb_pop; i++) {
                    N[i] = N[0];
                }
            }
  
            // Cas de tailles heterogenes
            if (rep_tailles.equals("NH")) {
                /* Dans le cas de tailles heterogenes, le separateur utilise est
                  la tabulation*/
  /*               st = new StringTokenizer(entree.readLine(), "\t");
                for (int i = 0; i < nb_pop; i++)
                    N[i] = Integer.valueOf(st.nextToken()).intValue();
            }
   
            // Autres cas
            if ((!rep_tailles.equals("H")) && (!rep_tailles.equals("NH"))) {
                System.out.println("Error of file data entry:" +
                        rep_tailles);
                System.exit(1);
            }
   
            nbcop = new float[nb_pop];
            iles = new Population[nb_pop];
            st = new StringTokenizer(entree.readLine(), "\t");
   
            /* saisie du nombre
            moyen de copies par population*/
  /*           for (int i = 0; i < nb_pop; i++)
                nbcop[i] = Float.valueOf(st.nextToken()).floatValue();
   
            // Initialisation des iles
            setPopulations(entree);
   
            imigration = new int[nb_pop][nb_pop - 1];
            voisinage = new Population[nb_pop][nb_pop - 1];
            tx_migration = new float[nb_pop][nb_pop];
            for (int i = 0; i < nb_pop; i++)
                tx_migration[i][i] = (float) N[i];
            rep_tx = entree.readLine();
   
            // Cas d'une diffusion homogene
            if (rep_tx.equals("H")) {
                tx_migration[1][0] =
                        Float.valueOf(entree.readLine()).floatValue();
                for (int i = 0; i < nb_pop; i++) {
                    for (int j = 0; j < i; j++) {
                        tx_migration[i][j] = tx_migration[1][0];
                        tx_migration[j][i] = tx_migration[i][j];
                        tx_migration[i][i] -= tx_migration[i][j];
                        tx_migration[j][j] -= tx_migration[j][i];
                    }
                }
            }
   
            // Cas d'une diffusion heterogene
            if (rep_tx.equals("NH")) {
                for (int i = 1; i < nb_pop; i++) {
                    st = new StringTokenizer(entree.readLine(), "\t");
                    for (int j = 0; j < i; j++) {
                        tx_migration[i][j] = Float.valueOf(
                                st.nextToken()).floatValue();
                        tx_migration[j][i] = tx_migration[i][j];
                        tx_migration[i][i] -= tx_migration[i][j];
                        tx_migration[j][j] -= tx_migration[j][i];
                    }
                }
            }
   
            // Autres cas
            if ((!rep_tx.equals("H")) && (!rep_tx.equals("NH"))) {
                System.out.println("Erreur");
                System.exit(1);
            }
   
            entree.close();
   
        }
   
        catch (IOException e) {
            e.printStackTrace();
        }
   
        this.setNeightbourhood();
   
        return;
    }*/
