package projet_these.et;

import projet_these.es.*;
import projet_these.ig.*;
import java.io.*;
import java.util.*;




/**
 * Set of tools for IslandModel
 * @author Gregory Deceliere - UMR CNRS 5558 - "BBE"
 * @version 26/06/02
 */
public class Util_Islands extends IslandModel
{


    /**
     * Contructor of initialization of the model
     */
    public Util_Islands()
    {
        super();
    }


    /**
        * Procedure of simulation starting from the OS console
        * @param    titre   The software name
        * @param    fenetre The gui main frame
        */
    public void consoleDOS(String titre, MainFrame fenetre)
    {

        // Number of the menu choice
        int choix = 0;

        // Consecutives simulations number
        int nb_simul = 0;

        // Answer for the running of simulation
        String rep_thread = "";

        // Affichage du Menu
        System.out.println("\n" + titre + "\n");
        System.out.println("Procedures\n");
        System.out.println("\t[1]-Start the GUI (Graphical User Interface)");
        System.out.println("\t[2]-File data entry mode");
        System.out.println("\t[3]-Run");
        System.out.println("\t[4]-Exit\n");
        System.out.print("Enter your choice : ");

        // Choice capture in a do while fro error managment
        do
        {
            try
            {
                choix = ReadStandardInput.readInteger();
            }
            catch (IOException e)
            {
                System.out.println(e);
            }
            if (choix != 1 && choix != 2 && choix != 3 && choix != 4)
                System.out.println("Error of data entry:enter 1,2,3 or 4");
        }
        while (choix != 1 && choix != 2 && choix != 3 && choix != 4)
            ;

        // Multiple choice selection
        switch (choix)
        {

            // Parameters in line capture
            case (1):

                if (!fenetre.isShowing())
                    fenetre.setVisible(true);

                break;

            // Input parameters file capture
            case (2):

                if (fenetre.isShowing())
                    fenetre.setVisible(false);

                System.out.print("\nWhat is the name of the input file ? ");
                try
                {
                    nomFic1 = ReadStandardInput.readString();
                }
                catch (IOException e)
                {
                    System.out.println(e);
                }

                System.out.print("\nWhat is the name of the output file ? ");
                try
                {
                    nomFic2 = ReadStandardInput.readString();
                }
                catch (IOException e)
                {
                    System.out.println(e);
                }
                InitIslanModelFromFile.initModel(this);

                break;

            // Run
            case (3):

                if (fenetre.isShowing())
                    fenetre.setVisible(false);

                // Simulations number choice
                System.out.print("How many simulations do you wish ? ");
                try
                {
                    nb_simul = ReadStandardInput.readInteger();
                }
                catch (IOException e)
                {
                    System.out.println(e);
                }

                // Running confirmations
                do
                {
                    rep_thread = "";
                    System.out.print("Input :" + nomFic1 +
                            "\nOutput :" + nomFic2 +
                            "\n Number of simulations :" + nb_simul +
                            "\nWould you want to launch the thread (y/n)?");
                    try
                    {
                        rep_thread = ReadStandardInput.readString();
                    }
                    catch (IOException e)
                    {
                        System.out.println(e);
                    }
                }
                while (!rep_thread.equals("y") && !rep_thread.equals("n"))
                    ;

                // Runnings of simulations one after one
                if (rep_thread.equals("y"))
                {
                    nomFic2 = (nomFic2.endsWith(".txt")) ?
                            nomFic2.substring(0,
                            nomFic2.indexOf(".txt")) + "simul" :
                            nomFic2 + "simul";

                    try
                    {
                        CopyAndPasteIslandModel.copyIslandModel(this);
                    }
                    catch(IOException ioexc)
                    {};

                    for (int i = 1; i <= nb_simul; i++)
                    {
                        /*try
                        {
                            //this.buildModel(CopyAndPasteIslandModel.pasteIslandModel());
                        }
                        catch(IOException ioexc)
                        {};*/
                        nomFic2 = nomFic2.substring(0,
                                nomFic2.indexOf("simul") + 5) + i + ".txt";
                        System.out.println("\nprocess " + i + "\n");
                        // TODO
                        //this.simulation();
                        for (int t = 0; t <= this.tps; t++)
                            this.simulation(t);
                    }
                    if(this.modelFile != null)
                        this.modelFile.delete();
                }

                break;

                // Leaving from program
            case (4):

                System.exit(1);
                break;
        }

        return;
    }


    /**
        * Procedure modifying the standard model
        * starting from a model passed in argument
        * @param    model   The model used for copy
        */
    public void buildModel(IslandModel model)
    {
        this.conta_mod = model.conta_mod;
        this.hot_spots = model.hot_spots;
        this.iles = model.iles;
        this.imigration = model.imigration;
        this.modelFile = model.modelFile;
        this.N = model.N;
        this.nb_pop = model.nb_pop;
        this.nbcop = model.nbcop;
        this.nbcop2 = model.nbcop2;
        this.nomFic1 = model.nomFic1;
        this.nomFic2 = model.nomFic2;
        this.rep_tailles = model.rep_tailles;
        this.rep_tx = model.rep_tx;
        this.tps = model.tps;
        this.tx_migration = model.tx_migration;
        this.voisinage = model.voisinage;
    }


    /**
     * Static procedure testing if a String represent a number
     * @param s String tested
     * @return Boolean 
     */
    public static boolean isNumeric(String s)
    {

        for (int i = 0; i < s.length(); i++)
        {
            char c = s.charAt(i);
            if (! (Character.isDigit(c)))
                return false;
        }
        return true;
    }


    /**
     * Static procedure testing if a String represent a positive float
     * @param s String tested
     * @return Boolean
     */
    public static boolean isPositiveFloat(String s)
    {

        int k = 0;
        for (int i = 0; i < s.length(); i++)
        {
            char c = s.charAt(i);
            if (new Character(c).toString().equals("."))
                ++k;
            if ((Character.getNumericValue(c) < 0 &&
                    ! (new Character(c).toString().equals("."))) ||
                    Character.getNumericValue(c) > 9 || k > 1)
                return false;
        }
        return true;
    }
}



