/*
 * Coordinates.java
 *
 * Created on 14 mars 2005, 23:45
 */

package projet_these.ig;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.table.*;
import java.awt.geom.*;
import java.io.*;
import java.util.*;

/**
 * The class instantiated by MapSetting when the user click on "New Entry".
 * Allow the user to define custom coordinates in the form af a JTable and to save 
 * them in a .coo file.
 * The Maps xData and yData field are then updaten
 * @author Yann Letrillard
 */
public class Coordinates extends JDialog {
    
    /**
     * The button that permit the user to save the coordinates in a .coo file
     */
    private JButton jButton1;
    /**
     * The button that permit the user to confirm the coordinates and thus transmit
     * them to Maps.
     */
    private JButton jButton2;
    /**
     * The button that permit the user to cancel.
     */
    private JButton jButton3;
    /**
     * The panel positionated at the bottom of the JDialog Coordinates containing the 
     * 3 buttoms.
     */
    private JPanel jPanel1;
    /**
     * The scroll pane containing the JTable.
     */
    private JScrollPane jScrollPane1;
    /**
     * The JTable by which the user fills and see the coordiantes.
     */
    private JTable jTable1;
    /**
     * The Maps intance
     */
    private Maps map;
    /**
     * The table containing the actual coordinates of Maps
     */
    private Object [][]data_table;
    
    
    /**
     * Constructs the JDialog coordinates editor
     * @param parent The MaFenetre instance used to create the JDialog, centered.
     * @param map The Maps instance to which actual coordinates are get and/or set
     * @param modal Boolean Indicate if the Jdialog is always on top
     */
    public Coordinates(Frame parent, Maps map, boolean modal) {
        
        super(parent, modal);
        
        setMap(map);
        
        setDataTable();
        
        initComponents();
        
        setLocation((int)(getParent().getWidth()/1.5), (int)(getParent().getHeight()/1.5));
        
    }
    
    /**
     * Construct the graphics components of the coordinates editor
     */
    private void initComponents() {
        
        jPanel1 = new JPanel();
        jButton1 = new JButton();
        jButton2 = new JButton();
        jButton3 = new JButton();
        jScrollPane1 = new JScrollPane();
        jTable1 = new JTable();
        
        setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
        setTitle("Coordinates editor");
        setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
        setModal(true);
        setResizable(true);
        
        
        /*
         *Setting of the Jpanel1 containing the buttons with their respective action manager
         */
        jPanel1.setLayout(new FlowLayout(FlowLayout.RIGHT));
        jButton1.setFont(new Font("Dialog", 0, 12));
        jButton1.setText("Save");
        jButton1.setEnabled(true);
        jButton1.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                jButton1ActionPerformed(evt);
            }
        });
        jPanel1.add(jButton1);
        
        jButton2.setFont(new Font("Dialog", 0, 12));
        jButton2.setText("OK");
        jButton2.setEnabled(true);
        jButton2.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                jButton2ActionPerformed(evt);
            }
        });
        jPanel1.add(jButton2);
        
        jButton3.setFont(new Font("Dialog", 0, 12));
        jButton3.setText("Cancel");
        jButton3.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                jButton3ActionPerformed(evt);
            }
        });
        jPanel1.add(jButton3);
        getContentPane().add(jPanel1, BorderLayout.SOUTH);
        
        
        /*
         *Setting of the JTable by wich coordinates will be set
         */
        
        jScrollPane1.setPreferredSize(new Dimension(253, 100));
        
        //Setting of the JTable
        jTable1.setModel(new javax.swing.table.DefaultTableModel(
                
                //contain the coordiantes
                data_table,
                
                //gives the names of columns
                new String [] {"Population n", "X", "Y"}) 
                //determines the properties of each cells
                {
                    //gives the type of class accepted in the JTable
                    Class[] types = new Class [] {
                        String.class, Double.class, Double.class
                    };
                    //Set the editable columns
                    boolean[] canEdit = new boolean [] {
                        false, true, true
                    };
                    
                    public Class getColumnClass(int columnIndex) {
                        return types [columnIndex];
                    }
                    
                    public boolean isCellEditable(int rowIndex,
                            int columnIndex) {
                        return canEdit [columnIndex];
                    }
                });
                
                jScrollPane1.setViewportView(jTable1);
                getContentPane().add(jScrollPane1, BorderLayout.CENTER);
                
                pack();
    }
    
    /**
     * The action associated to jButton1 in order to save
     * @param evt Saving of the coordinates
     */
    private void jButton1ActionPerformed(ActionEvent evt) {
        
        this.uneditingRegularCells();
        
        //If the table is not filled correctly
        if (jTable1.isEditing()){
            JOptionPane.showMessageDialog(map.getSuperFrame(),
                    "Error in datas? Verify your entries\nor verify if a cell is not selected", "",
                    JOptionPane.ERROR_MESSAGE);
        } 
        
        //else a JOptionPane will ask a file name to save the coordinates
        else {
            String file_name = JOptionPane.showInputDialog(map.getSuperFrame(),
                    "Enter the name (with path?) of the file to save coordinates", "",
                    JOptionPane.QUESTION_MESSAGE);
            try {
                saveCoordinates(file_name);
            }catch (IOException e){
            };
        }
    }

    /**
     * The action associated to jButton2 in order to cancel and close the JDialog
     * @param evt Canceling
     */
    private void jButton3ActionPerformed(ActionEvent evt) {
        dispose();
    }
    
    
    /**
     * The action associated to jButton2 in order to confirm possible changes and quit 
     * the editor
     * @param evt Transmission to Maps of the new coordinates and leaving of the editor
     */
    private void jButton2ActionPerformed(ActionEvent evt) {
        
        this.uneditingRegularCells();
        
        //If the table is not filled correctly
        if (jTable1.isEditing()){
            JOptionPane.showMessageDialog(map.getSuperFrame(),
                    "Error in datas? Verify your entries\nor verify a cell is not selected", "",
                    JOptionPane.ERROR_MESSAGE);
        } 
        
        //the coordinates are transmitted to Maps
        else {
            setMapCoordinates();
            dispose();
        }
    }
    
    
    /**
     * Transmit to Maps a new List of coordinates filled from the JTable
     */
    private void setMapCoordinates(){
        
        //list of the same format than in Maps
        java.util.List all_coordinates = new ArrayList();
        
        //Filing of the list from the JTbale entries
        for (int i=0; i < map.getTableWidth()-1; i++){
            all_coordinates.add(new Double[]{
                (Double)jTable1.getValueAt(i, 1),
                (Double)jTable1.getValueAt(i, 2)
            });
        }
        map.setCoordinates(all_coordinates);
    }
    
    /**
     * Set the Maps instance from the argument of the constructor of Coordinates.
     * Will be used to transmit or get the coordinates to/from map
     * @param map The Maps instance
     */
    private void setMap(Maps map){
        this.map = map;
    }
    
    /**
     * Set data_set field of Coordinates from the coordinates of map
     */
    private void setDataTable(){
        
        //The data table used to fill the JTable from Maps' coordinates
        this.data_table = new Object[map.getTableWidth() - 1][3];
        //The first column contains the number of populations, the orthers the coordinates
        for(int i=0; i<map.getTableWidth() - 1; i++){
            this.data_table[i][0]="" + (i+1);
            this.data_table[i][1] = map.xData[i];
            this.data_table[i][2] = map.yData[i];
        }
    }
    
    /**
     * Saves the coordinates filled in the JTable to a .coo file of the name set by the 
     * user
     * @param file_name The String containing the name of the file the user set in 
     * jButton1ActionPerformed method
     * @throws java.io.IOException If problems occurs with the fileWriter used
     */
    private void saveCoordinates(String file_name) throws IOException {
        
        String correct_file_name = "";
        
        //If the user haven't add the extension it is added
        correct_file_name = (file_name.endsWith(".coo")) ?
            correct_file_name : (file_name + ".coo");
        
        //A file is created from the given name with a file writer
        File file = new File(correct_file_name);
        FileWriter file_writer = new FileWriter(file);
        
        //Writing of the header of the file
        file_writer.write("\t" + "X\t" + "Y\n");
        
        //Writing of the coordinates from the JTable
        for (int i = 0; i < map.getTableWidth() - 1; i++){
            file_writer.write("" + data_table[i][0] + "\t"
                    + data_table[i][1] + "\t" + data_table[i][2] + "\n");
        }        
        file_writer.close();
    }
    
    private void uneditingRegularCells(){
        int col = this.jTable1.getEditingColumn();
        int row = this.jTable1.getEditingRow();
        if (col != -1 && row != -1) this.jTable1.getCellEditor(row, col).stopCellEditing();
    }
}
