/*************************************************************************
*         Copyright notice for JFreeChart and JCommon libraries          *
*                                                                        *
*The present application links to JFreeChart and JCommon libraries as    *
*external jar files that are under the GNU Lesser General Public License.*
*You can find a copy of that license at the root folder containing       *
*the main class of TESD. More informations such as complete source codes *
*to modify the libraries are available at this web site: www.jfree.org.  *
*************************************************************************/


/*
 * GraphicsDisplay.java
 *
 * Created on 1 juillet 2005, 00:21
 */

package projet_these.ig;

import java.awt.event.*;
import javax.swing.JOptionPane;

/**
 * When a simulation is done the user is convited to see the 
 * results as various graphics (maps, curves or histograms)
 * @author Yann Letrillard
 */
public class GraphicsDisplay extends javax.swing.JDialog implements ActionListener{
    
    /**
     * Creates new form GraphicsDisplay
     * @param parent The parent frame
     * @param modal Tels if this JDialog hae to be always on top 
     * and keep focus
     * @param super_frame The TESD main frame
     * @param simul_name The simulation name used to create the ouputfiles
     */
    public GraphicsDisplay(java.awt.Frame parent, boolean modal, MainFrame super_frame, String simul_name) {
        super(parent, modal);
        this.super_frame = super_frame;
        this.simul_name = simul_name;
        initComponents();
        jButton1.setActionCommand("Curves");
        jButton2.setActionCommand("Histograms");
        jButton3.setActionCommand("Maps");
        jButton4.setActionCommand("MapsAnim");
        jButton2.addActionListener(this);
        jButton1.addActionListener(this);
        jButton3.addActionListener(this);
        jButton4.addActionListener(this);
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        java.awt.GridBagConstraints gridBagConstraints;

        jLabel1 = new javax.swing.JLabel();
        jPanel1 = new javax.swing.JPanel();
        jButton1 = new javax.swing.JButton();
        jButton2 = new javax.swing.JButton();
        jButton3 = new javax.swing.JButton();
        jButton4 = new javax.swing.JButton();

        getContentPane().setLayout(new java.awt.GridBagLayout());

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setTitle("GRAPHICS");
        jLabel1.setText("What kind of graphic do you want to display?");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.insets = new java.awt.Insets(10, 10, 0, 10);
        getContentPane().add(jLabel1, gridBagConstraints);

        jButton1.setText("Curves");
        jPanel1.add(jButton1);

        jButton2.setText("Histograms");
        jPanel1.add(jButton2);

        jButton3.setText("Maps");
        jPanel1.add(jButton3);

        jButton4.setText("Animated Map");
        jPanel1.add(jButton4);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new java.awt.Insets(10, 10, 10, 10);
        getContentPane().add(jPanel1, gridBagConstraints);

        pack();
    }
    // </editor-fold>//GEN-END:initComponents
    
    /**
     * What to do when the user validate its choice (show a curve, map, 
     * histograms or dispose)
     * @param e clicking on ok or cancel
     */
    public void actionPerformed(ActionEvent e) {
        if (e.getActionCommand() == "Curves"){
            Object[] options = {"Variances", "Means"};
            int reponce = JOptionPane.showOptionDialog(this, "Do you want to see the curve for:", "Kind of curve", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE, null, options, options[1]);
            String extention = (reponce == 1) ? ".mea" : ".var";
            Curves curve = new Curves(super_frame, simul_name + extention);
            if (curve.dispose == false){
                curve.setVisible(true);
                super_frame.desktop.add(curve);
                try {
                    curve.setSelected(true);
                } catch(java.beans.PropertyVetoException E) {
                }
            }
        }
        
        if (e.getActionCommand() == "Histograms"){
            Histograms histogram = new Histograms(super_frame, simul_name + ".frq");
            if (histogram.dispose == false){
                histogram.setVisible(true);
                super_frame.desktop.add(histogram);
                try {
                    histogram.setSelected(true);
                } catch(java.beans.PropertyVetoException E) {
                }
            }
        }
        
        if (e.getActionCommand() == "Maps"){
            
            Graphiques map = new Maps(super_frame, simul_name + ".mea");
            //If map settings was well accomplished, map was setted visible so
            //map can be displayed
            if (map.dispose == false){
                map.setVisible(true);
                super_frame.desktop.add(map);
                try {
                    map.setSelected(true);
                } catch(java.beans.PropertyVetoException E) {
                    E.printStackTrace();
                }
            }
        }
        
        if (e.getActionCommand() == "MapsAnim"){
             Graphiques mapanim = new MapsAnim(super_frame, simul_name + ".mea");
            //If map settings was well accomplished, map was setted visible so
            //map can be displayed
            if (mapanim.dispose == false){
                mapanim.setVisible(true);
                super_frame.desktop.add(mapanim);
                try {
                    mapanim.setSelected(true);
                } catch(java.beans.PropertyVetoException E) {
                    E.printStackTrace();
                }
            }
        }
    }
    
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton jButton1;
    private javax.swing.JButton jButton2;
    private javax.swing.JButton jButton3;
    private javax.swing.JButton jButton4;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JPanel jPanel1;
    // End of variables declaration//GEN-END:variables
    private MainFrame super_frame;
    private String simul_name;
}
