/*************************************************************************
*         Copyright notice for JFreeChart and JCommon libraries          *
*                                                                        *
*The present application links to JFreeChart and JCommon libraries as    *
*external jar files that are under the GNU Lesser General Public License.*
*You can find a copy of that license at the root folder containing       *
*the main class of TESD. More informations such as complete source codes *
*to modify the libraries are available at this web site: www.jfree.org.  *
*************************************************************************/


package projet_these.ig;

import javax.swing.*;
import java.awt.event.*;
import java.awt.*;
import org.jfree.chart.*;
import org.jfree.chart.plot.*;
import org.jfree.data.xy.*;
import org.jfree.data.xy.AbstractIntervalXYDataset.*;
import org.jfree.data.statistics.*;
import org.jfree.chart.axis.ValueAxis.*;
import org.jfree.data.*;

/**
 * Title : Histograms
 * Description : A frame showing as histograms transposition frequencies
 * Copyright :    Copyright (c) 2004
 * Society : CNRS UMR 5558
 * @author Gregory Deceliere
 * @version 1.0
 */

public class Histograms extends Graphiques {
    
    /**
     * The number of bin chosen by the user (a bin corespond to the class width of an 
     * histogram).
     */
    protected int bin_number;
    
    /**
     * The Panel Containing all the histograms chart
     */
    protected JScrollPane all_histograms_panel;
    
    /**
     * The width of each histograms
     */
    private static int chart_width = 487;
    
    /**
     * The eight of each histograms
     */
    private static int chart_height = 327;
    
    /**
     * The number of histograms corresponding to the number of popualtions from outputable
     */
    protected static int chart_number;
    
    
    /**
     * Constructs a JInternalFrame containing all the histograms representations of 
     * insertion frequency distribution of each popualtions
     * @param output_file_name The file containing data (average insertion frequencies along 
     * chromosomes) from a simulation to plot as histograms
     * @param super_frame The MainFrame main frame used by the default constructor.
     */
    public Histograms(MainFrame super_frame, String output_file_name) {
        
        super(super_frame, "Histogram", true, output_file_name);
        
        //If the choice of the file was successful, Histogram is fully constructed
        if (this.dispose == false){
            
            this.chooseBinNumber();
            
            this.setAutoscrolls(true);
            
            this.getContentPane().setLayout(new BorderLayout());
            
            this.setAndShowAllHistogramPanel();
        }
    }
    
    /**
     * Popups a JOptionPane in order to get the user bin number's choice (default is 50)
     */
    protected void chooseBinNumber() {
        
        //The 3 possibilites proposed to the user
        Integer[] possibilities = {new Integer(11), new Integer(20), new Integer(50)};
        
        //An empty icon needed by the JOptionPane
        Icon void_icon = new ImageIcon();
        
        //The JOptionPane is popuped and the user's choice retained in choice
        Integer choice = (Integer)JOptionPane.showInputDialog(getSuperFrame(),
                "Select a custom number of histogram bin\n"+"(default is 50)", "",
                JOptionPane.QUESTION_MESSAGE,
                void_icon,
                possibilities,
                possibilities[2]);
        
        //Setting of the Map's bin number if different to the default one
        this.bin_number = (choice != possibilities[2]) ? choice.intValue() : possibilities[2].intValue();
    }
    
    
    /**
     * Creates a Panel to which all the histogramChart will be added and put it in a 
     * JScrollPane that is thus added to the ContentPane of Histograms
     */
    protected void setAndShowAllHistogramPanel() {
        //Note the difference between all_histogram_panel & all_histograms_panel
        //The panel that will be added to the jscroll pane containing all the ChartPanel
        JPanel all_histogram_panel = new JPanel();
        
        //The ChartPanels are added to all_histogram_panel
        for (int j=1; j < getTableWidth(); j++) {
            all_histogram_panel.add(createHistogramPanel(j));
        }
        
        //Determination of the all_histogram_panel in order all ChartPanel can be visible in it
        this.chart_number = this.getTableWidth()-1;
        //A factor giving the number of lines of ChartPanel
        int factor = (chart_number % 2 != 0) ? (chart_number/2)+1 : (chart_number/2);
        //Setting of the prefered size
        //all_histogram_panel.setPreferredSize(new Dimension(this.getWidth(), factor * (chart_height+5)));
        
        //all_histogram_panel is added to all_histograms_panel
        all_histograms_panel = new JScrollPane(all_histogram_panel);
        
        //all_histograms_panel is added to the maps main panel
        this.getContentPane().add(all_histograms_panel, BorderLayout.CENTER);
    }
    
    /**
     * Creates a ChartPanel to which the histogram of the j popualtion is added from 
     * createHistogram method
     * @param j The number of the popualtion to which a histogram have to be done
     * @return The ChartPanel that was created
     */
    protected ChartPanel createHistogramPanel(int j) {
        
        //Construction of a ChartPanel with the currently created chart histogram from the jth popopualtion
        ChartPanel chart_panel = new ChartPanel(createHistogram(j));
        
        //Set the possibility to zoom in the plot with transparant box
        chart_panel.setMouseZoomable(true, false);
        
        chart_panel.setPreferredSize(new Dimension(this.chart_width,this.chart_height));
        
        return chart_panel;
    }
    
    /**
     * Creates an histogram JFreeChart for the given population from the corresponding
     * dataset
     * @param j The number of the popualtion to which a histogram have to be done
     * @return The histogram JFreeChart that was created
     */
    protected JFreeChart createHistogram(int j) {
        
        //The dataset for this population
        HistogramDataset dataset = setHistogramDataset(j);
        
        //Creates an Histogram JFreeChart with the specified labels, the dataset, a VERTICAL
        //PlotOrientation (XAxis or also range Axis corresponds insertion site), a legend, a
        //usefull JFreeCart tooltip and without HTML properties.
        String title = output_file_name.substring(output_file_name.lastIndexOf(System.getProperty("file.separator"))+1) + " - Population " + (j);
        JFreeChart histogram = ChartFactory.createHistogram(title,
                "insertion frequency",
                "frequency",
                dataset,
                PlotOrientation.VERTICAL,
                false,
                true,
                false);
        
        return histogram;
    }
    
    /**
     * Creates the HistogramDataset from the output table
     * @param j The number of the popualtion to which a dataset have to be done
     * @return the HistogramDataset that was created
     */
    protected HistogramDataset setHistogramDataset(int j) {
        //The genome's site or the line in the output table
        int i;
        
        //Dynamic naming of the jth population
        String population_name = "Population" + j;
        
        //The table containing one population to be added to the dataset
        double the_j_population_series[] = new double[getTableHeight()];
        
        //Filling of the XYSeries from table for the jth population
        for (i=0; i < getTableHeight()-1; i++) {
            the_j_population_series[i] = output_table[i][j];
        }
        
        //The dataset in the format needed by the Histogram JFreeChart
        HistogramDataset dataset = new HistogramDataset();
        dataset.addSeries(population_name, the_j_population_series, bin_number);
        
        return dataset;
    }
    
    
}