/*************************************************************************
*         Copyright notice for JFreeChart and JCommon libraries          *
*                                                                        *
*The present application links to JFreeChart and JCommon libraries as    *
*external jar files that are under the GNU Lesser General Public License.*
*You can find a copy of that license at the root folder containing       *
*the main class of TESD. More informations such as complete source codes *
*to modify the libraries are available at this web site: www.jfree.org.  *
*************************************************************************/

package projet_these.ig;

import javax.swing.*;
import java.awt.*;
import org.jfree.chart.*;
import org.jfree.chart.plot.*;
import org.jfree.data.contour.*;
import org.jfree.data.*;
import org.jfree.data.xy.*;
import java.awt.event.*;
import java.lang.*;
import java.lang.Number.*;
import java.util.*;
import java.lang.Math.*;
import java.awt.geom.*;
import org.jfree.chart.renderer.xy.*;
import org.jfree.chart.annotations.*;

/**
 * A Graphiques inherited class used to plot the .var and .mea files as maps.
 * @author Gregory Deceliere
 * @version 1.0
 */

public class Maps extends Graphiques {
    
    /**
     * The number of generation separating each representations
     */
    public int interval;
    /**
     * The number of chart that will be painted according to the interval value and the
     * number of generations
     */
    public int chart_number;
    /**
     * The table containing the absciss of the popualtions
     */
    protected  Double[] xData;
    /**
     * The table containing the ordinates of the popualtions
     */
    protected  Double[] yData;
    /**
     * A table containg the mean or variances values at each intervals of all populations
     */
    protected  Double[] zData;
    /**
     * The maximum absciss among popualtions'ones. Used to scale the plot area
     */
    private double x_max;
    /**
     * The maximum ordinate among popualtions'ones. Used to scale the plot area
     */
    private double y_max;
    /**
     * The minimum absciss among popualtions'ones. Used to scale the plot area
     */
    private double x_min;
    /**
     * The minimum ordinate among popualtions'ones. Used to scale the plot area
     */
    private double y_min;
    /**
     * The width of the charts
     */
    private static int chart_width = 487;
    /**
     * The height of the charts
     */
    private static int chart_height = 440;
    
    /**
     * Constructs a JInternalFrame containing all the map representations of popualtions
     * at given interval and coordinates
     * @param output_file_name The file containing data (means of variances of TE 
     * copies number) to be represented as maps
     * @param super_frame The MainFrame main frame used by the default constructor.
     */
    public Maps(MainFrame super_frame, String output_file_name) {
        
        super(super_frame, "Maps", false, output_file_name);
        
        //The JDialog by which Maps's is set.
        MapSettings maps_settings;
        if (output_file_name == ""){
            //The JDialog by which Maps's is set. true determine if it's modal or not
            maps_settings = new MapSettings(getSuperFrame(), this, true, true);
            maps_settings.setVisible(true);
        }
        if (output_file_name != ""){
            maps_settings = new MapSettings(getSuperFrame(), this, true, false);
            maps_settings.setVisible(true);
        }
        
        
        //if the setting was successfull map is fully constructed
        if (this.dispose == false) {
            //createAllMapsPanel return the JScrollPane containing all the maps
            //after it called all the various fonctions needed
            this.getContentPane().add(this.createAllMapsPanel());
        }
    }
    
    /**
     * The methods that creates a panel to which all the mapCharts will be added and then
     * add it to a scroll pane that will be returned.
     * @return A JScrollPane that contain all the mapCharts
     */
    protected JScrollPane createAllMapsPanel(){
        
        //The panel in wich all maps will lay
        JPanel all_maps = new JPanel();
        
        //The time in generations at which a map is done
        int time = 0;
        
        //At each give interval, a map chart is created and added to all_maps
        for(int j=0; j<chart_number; j++){
            all_maps.add(createMapChart(time));
            time += this.interval;
        }
        
        //A factor giving the number of lines of ChartPanel
        int factor = (chart_number % 2 != 0) ? (chart_number/2)+1 : (chart_number/2);
        //Setting of the prefered size
        all_maps.setPreferredSize(new Dimension(this.getWidth(), factor * (chart_height+5)));
        
        JScrollPane scroll_pane = new JScrollPane(all_maps);
        return scroll_pane;
    }
    
    /**
     * Create a BubbleChart JFreeChart at the given time (a map) after calling the
     * createContourDataset method. Then it adds the chart to the ChartPanel
     * @param time The generation at which the means or variances values will be used to set the
     * zData table
     * @return The ChartPanel that contains the map
     */
    protected ChartPanel createMapChart(int time){
        
        //Setting of the dataset of this current chart
        AbstractXYZDataset dataset = createContourDataset(time);
        
        //Set the title of the current map: "File name(without path) - (current)Time"
        String title = output_file_name.substring(output_file_name.lastIndexOf
                (System.getProperty("file.separator"))+1) + " - Time " + time;
        
        //Creates a Bubbe JFreeChart with the specified labels, the dataset, a VERTICAL
        //PlotOrientation (XAxis or also domain Axis corresponds insertion site), no legend, a
        //usefull JFreeCart tooltip and without HTML properties.
        JFreeChart map = ChartFactory.createBubbleChart(title, "X", "Y", dataset,
                PlotOrientation.VERTICAL, false, true, false);
        
        //Adds the name of population on first chart
        setChartLabels(map);
        
        //Calls of the function that custom axis range
        setChartAxis(map);
        
        //The container of map
        ChartPanel map_panel = new ChartPanel(map);
        
        //Zoom in the map is possibe
        map_panel.setMouseZoomable(false, false);
        
        //The width is setted to allow to fill 2 chart in one line of Maps JInternalFrame
        //thus the height is adjusted to have square shape of plot
        map_panel.setPreferredSize(new Dimension(chart_width,chart_height));
        
        //Returns of the setted new chart
        return map_panel;
    }
    
    /**
     * Add the "Pop X" labels of each populations upper on the right of the center of
     * population's circles
     * @param map The ChartPanel from which a the XYPlot will recieve the label
     */
    protected void setChartLabels(JFreeChart map){
        
        //for each population
        for (int i=0; i<getTableWidth()-1; i++){
            //"Pop x" is added at the upper right corner as XYTextAnnotation
            double label_inset = 0.25;
            XYTextAnnotation pointer = new XYTextAnnotation(
                    "Pop" + (i+1), xData[i].doubleValue() +label_inset, yData[i].doubleValue() + label_inset);
            map.getXYPlot().addAnnotation(pointer);
        }
    }
    
    /**
     * Set the rescaling of the axis when an unzoom is done
     * @param map The ChartPanel from which a the XYPlot will be properly rescalled
     */
    protected void setChartAxis(JFreeChart map){
        
        //Domain axis for VERTICAL PlotOrientation is the absis axis so Xaxis
        //Setted to the max Value of xData
        map.getXYPlot().getDomainAxis().setRange(new Range(0d, x_max),true,true);
        map.getXYPlot().getRangeAxis().setRange(new Range(0d, y_max),true,true);
    }
    
    /**
     * Creates a contour dataset used by the ChartFactory to plot the populations at the
     * coordinates given in xData and yData and as circles of the size given by zData.
     * @param time The generation at which the dataset is done
     * @return The AbstractXYZDataset created
     */
    protected AbstractXYZDataset createContourDataset(int time){
        //
        this.setzData(time);
        DefaultContourDataset dataset = new DefaultContourDataset("test", xData, yData, zData);
        return dataset;
    }
    
    /**
     * Set random coordinates as a List containing from an array of positions computed
     * with the square root of the popuation number and sent it to the setCoordinates method
     */
    protected void autoSetCoordinates(){
        
        Integer pop_number = new Integer(getTableWidth()-1);
        
        //The square root of pop_number rounded to the upper double
        double square = java.lang.Math.ceil(java.lang.Math.sqrt(pop_number.doubleValue()));
        
        //The total number of positions in the array
        Double positions_number = new Double(Math.pow((square), 2d));
        
        //The list of map that will contain all the coordinates populations can take
        java.util.List all_coordinates = new ArrayList(positions_number.intValue());
        
        //The initial coordinates
        double x=1d, y=1d;
        
        //Filling of the list of all the possible coordinates:
        //at each columns
        for (int i=0; i<square; i++){
            
            //at each lines the positions are calculated
            for(int j=0; j<square; j++){
                all_coordinates.add(new Double[]{new Double(x), new Double(y)});
                x +=1d;
            }
            x=1d;
            y +=1d;
        }
        
        //Mixing of the coordinates
        java.util.Collections.shuffle(all_coordinates);
        
        //The x & y_Data tables will be filled from all_coordinates
        this.setCoordinates(all_coordinates);
    }
    
    /**
     * Fills the xData and yData tables from the List
     * @param all_coordinates A List containing the coordinates as tables of size 2 containing the x and y
     * coordinates of each popualtions.
     */
    protected void setCoordinates(java.util.List all_coordinates){
        
        //The maximum & minimum coodrinates used to scale the plot area
        this.x_max = 0d;
        this.y_max = 0d;
        this.x_min = 0d;
        this.y_min = 0d;
        
        //The tables containg absis and ordinates
        this.xData = new Double[getTableWidth()-1];
        this.yData = new Double[getTableWidth()-1];
        
        
        for (int i=0; i<getTableWidth()-1; i++){
            //Filling of the tables from the list
            xData[i] = ((Double[])all_coordinates.get(i))[0];
            yData[i] = ((Double[])all_coordinates.get(i))[1];
            
            //Determination of the max or min coordinates
            x_max = (((Double[])all_coordinates.get(i))[0].doubleValue() > x_max) ? 
                ((Double[])all_coordinates.get(i))[0].doubleValue() : x_max;
            y_max = (((Double[])all_coordinates.get(i))[1].doubleValue() > y_max) ? 
                ((Double[])all_coordinates.get(i))[1].doubleValue() : y_max;
            x_min = (((Double[])all_coordinates.get(i))[0].doubleValue() < x_min) ? 
                ((Double[])all_coordinates.get(i))[0].doubleValue() : x_min;
            y_min = (((Double[])all_coordinates.get(i))[1].doubleValue() < y_min) ? 
                ((Double[])all_coordinates.get(i))[1].doubleValue() : y_min;
        }
        
        //The max values are increased in order to present a spaced map
        this.x_max += 1;
        this.y_max += 1;
        
        //Min values are also reduced on the conditions they'return not to small
        if (this.x_min >=2 && this.y_min >= 2){
            this.x_min -= 1;
            this.y_min -= 1;
        }
        
    }
    
    /**
     * Fills the zData table at the given number of generations from the .mea or .var
     * file.
     * @param time The number of generations at which a means or variances.
     */
    protected void setzData(int time){
        zData = new Double[getTableWidth()-1];
        //The factor by which zDatas have to be decreased in order bubbules don't overlap
        float factor = 2 / getTableMax();
        for (int i=1; i<getTableWidth(); i++){
            zData[i-1] = new Double(output_table[time][i] * factor);
        }
    }
    
    /**
     * Calculates the interval field of Maps in order 11 maps will be drawn by default
     * or computes an appropriate value in case of an inpertinent interval was entered
     * and then calculates corresponding number of charts
     * @param interval The interval value get from MapSettings jFormatetdTextFiled3
     */
    public void autoSetInterval(int interval){
        
        if (interval == -1){
            interval = (int)(java.lang.Math.floor((this.getTableHeight()-1)/10));
            this.setInterval(interval);
            setChartNumber(10);
        }  else if (interval > this.getTableHeight()-1){
            //interval is set to the max value
            this.setInterval(this.getTableHeight()-1);
            //chart_number is set to the min value that is the first and last generation
            this.setChartNumber(2);
        } else if (interval == 0){
            //interval is automatically setted to the min value
            this.setInterval(1);
            //chart_number is automatically setted to max graphic number that is the
            //number of generations + the first
            this.setChartNumber(this.getTableHeight());
        } else {
            //The entered value is setted to be map's interval
            this.setInterval(interval);
            //Calculation of graphics number:
            chart_number = (int)java.lang.Math.ceil((this.getTableHeight()-1)/interval);
            //Setting of map's charts number
            this.setChartNumber(chart_number);
        }
        
    }
    
    /**
     * Set the chart number field of Maps from the argument. Used by autoSetInterval
     * method
     * @param chart_number the number of chart to set as the field of Maps.
     */
    protected void setChartNumber(int chart_number){
        this.chart_number = chart_number + 1;
    }
    
    /**
     * Used by MapSettings to get the char number field of Maps and fill its jTextFiel4
     * @return The number of chart of Maps as int
     */
    protected int getChartNumber(){
        return this.chart_number;
    }
    
    /**
     * Used by autoSetInterval to set the interval field of Maps
     * @param interval the interval int value to set the corresponding field of Maps
     */
    protected void setInterval(int interval){
        this.interval = interval;
    }
    
    /**
     * Used by MapSettings to fill the value computed by autoSetInterval in
     * jFormatedTextField3
     * @return The interval number field of Maps as int
     */
    protected int getInterval(){
        return this.interval;
    }
    
}
